@@ -56,19 +56,13 @@ walk_unlinked_list(xfs_mount_t *mp, xfs_agnumber_t agno, xfs_agino_t start_ino)
 			 * inode.  if so, put it on the uncertain inode list
 			 * and set block map appropriately.
 			 */
-			if (find_inode_rec(agno, current_ino) == NULL)  {
+			if (find_inode_rec(mp, agno, current_ino) == NULL)  {
 				add_aginode_uncertain(agno, current_ino, 1);
 				agbno = XFS_AGINO_TO_AGBNO(mp, current_ino);
 
 				pthread_mutex_lock(&ag_locks[agno]);
-				switch (state = get_agbno_state(mp,
-							agno, agbno))  {
-				case XR_E_UNKNOWN:
-				case XR_E_FREE:
-				case XR_E_FREE1:
-					set_agbno_state(mp, agno, agbno,
-						XR_E_INO);
-					break;
+				state = get_bmap(agno, agbno);
+				switch (state) {
 				case XR_E_BAD_STATE:
 					do_error(_(
 						"bad state in block map %d\n"),
@@ -85,13 +79,12 @@ walk_unlinked_list(xfs_mount_t *mp, xfs_agnumber_t agno, xfs_agino_t start_ino)
 					 * anyway, hopefully without
 					 * losing too much other data
 					 */
-					set_agbno_state(mp, agno, agbno,
-						XR_E_INO);
+					set_bmap(agno, agbno, XR_E_INO);
 					break;
 				}
 				pthread_mutex_unlock(&ag_locks[agno]);
 			}
-			current_ino = dip->di_next_unlinked;
+			current_ino = be32_to_cpu(dip->di_next_unlinked);
 		} else  {
 			current_ino = NULLAGINO;;
 		}
@@ -114,23 +107,22 @@ process_agi_unlinked(xfs_mount_t *mp, xfs_agnumber_t agno)
 			XFS_AG_DADDR(mp, agno, XFS_AGI_DADDR(mp)),
 			mp->m_sb.sb_sectsize/BBSIZE, 0);
 	if (!bp)
-		do_error(_("cannot read agi block %lld for ag %u\n"),
+		do_error(_("cannot read agi block %" PRId64 " for ag %u\n"),
 			XFS_AG_DADDR(mp, agno, XFS_AGI_DADDR(mp)), agno);
 
 	agip = XFS_BUF_TO_AGI(bp);
 
-	ASSERT(no_modify || INT_GET(agip->agi_seqno, ARCH_CONVERT) == agno);
+	ASSERT(no_modify || be32_to_cpu(agip->agi_seqno) == agno);
 
 	for (i = 0; i < XFS_AGI_UNLINKED_BUCKETS; i++)  {
-		if (INT_GET(agip->agi_unlinked[i], ARCH_CONVERT) != NULLAGINO)  {
-			err += walk_unlinked_list(mp, agno, INT_GET(
-					agip->agi_unlinked[i], ARCH_CONVERT));
+		if (be32_to_cpu(agip->agi_unlinked[i]) != NULLAGINO)  {
+			err += walk_unlinked_list(mp, agno,
+					be32_to_cpu(agip->agi_unlinked[i]));
 			/*
 			 * clear the list
 			 */
 			if (!no_modify)  {
-				INT_SET(agip->agi_unlinked[i], ARCH_CONVERT,
-					NULLAGINO);
+				agip->agi_unlinked[i] = cpu_to_be32(NULLAGINO);
 				agi_dirty = 1;
 			}
 		}
